package cn.ecpark.service.usergw.biz.filters.factory;

import java.nio.ByteBuffer;
import java.util.Arrays;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;

import com.alibaba.fastjson.JSON;

import org.apache.dubbo.rpc.service.GenericService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cloud.gateway.filter.GatewayFilter;
import org.springframework.cloud.gateway.filter.GatewayFilterChain;
import org.springframework.cloud.gateway.filter.OrderedGatewayFilter;
import org.springframework.cloud.gateway.filter.factory.AbstractGatewayFilterFactory;
import org.springframework.context.ApplicationContext;
import org.springframework.core.Ordered;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpStatus;
import org.springframework.http.server.reactive.ServerHttpRequest;
import org.springframework.http.server.reactive.ServerHttpResponse;
import org.springframework.stereotype.Component;
import org.springframework.web.server.ServerWebExchange;

import cn.ecpark.service.usergw.biz.filters.bean.GenericServicePool;
import reactor.core.publisher.Mono;;

@Component
public class DubboGatewayFilterFactory extends AbstractGatewayFilterFactory<DubboGatewayFilterFactory.Config> {

    @Autowired
    private ApplicationContext appContext;

    /**
     * DUBBO_URI key.
     */
    public static final String DUBBO_URI = "dubbo_uri";
    public static HashMap<String, List<String>> mehtods = new HashMap<>();

    public DubboGatewayFilterFactory() {
        super(Config.class);
    }

    @Override
    public List<String> shortcutFieldOrder() {
        return Arrays.asList(DUBBO_URI);
    }

    @Override
    public GatewayFilter apply(Config config) {
        DubboGatewayFilter dubboGatewayFilter = new DubboGatewayFilter(config);
        return new OrderedGatewayFilter(dubboGatewayFilter, Ordered.HIGHEST_PRECEDENCE);
    }

    public static class Config {

        private String dubboUri;

        public String getDubboUri() {
            return dubboUri;
        }

        public void setDubboUri(String dubboUri) {
            this.dubboUri = dubboUri;
        }

    }

    public class DubboGatewayFilter implements GatewayFilter {

        private final Config config;
        private List<String> emptyList = new LinkedList<String>();

        public DubboGatewayFilter(Config config) {
            this.config = config;
        }

        @Override
        // @SuppressWarnings("unchecked")
        public Mono<Void> filter(ServerWebExchange exchange, GatewayFilterChain chain) {

            // TODO: 侵入私有变量, 但是这样不如直接改源码了, 效率低.
            // try {

            // Field filtersField = chain.getClass().getDeclaredField("filters");
            // filtersField.setAccessible(true);
            // ArrayList<GatewayFilter> filters =
            // (ArrayList<GatewayFilter>)filtersField.get(chain);
            // log.info(filters.toString());
            // } catch (Exception e) {
            // log.error(e.getMessage());
            // setResponseStatus(exchange, HttpStatus.INTERNAL_SERVER_ERROR);
            // return chain.filter(exchange);
            // }
            
            return chain.filter(exchange).then(Mono.fromRunnable(() -> {
                
                ServerHttpRequest req = exchange.getRequest();
                HttpHeaders headers = req.getHeaders();

                List<String> methodString = headers.get("method");
                List<String> params = headers.get("params");

                Object result = null;
                ServerHttpResponse response = exchange.getResponse();
                HttpStatus statusCode = response.getStatusCode();
                if (statusCode.value() == 200 && methodString.size() != 0) {
                    List<String> paramTypes;
                    // 判断全部函数允许, 必须带参数类型, 而且要匹配, 否则报错
                    if (config.dubboUri.charAt(5) == '-') {
                        paramTypes = headers.get("param-types");
                        if (paramTypes == null) {
                            paramTypes = emptyList;
                        }
                    } else {
                        paramTypes = mehtods.get(methodString.get(0));
                    }

                    if (paramTypes != null) {
                        int paramsSize = 0;
                        if (params != null) {
                            paramsSize = params.size();
                        }

                        if (paramTypes.size() == paramsSize) {

                            GenericServicePool gsPool = appContext.getBean(GenericServicePool.class);
                            GenericService gs = gsPool.get(config.dubboUri);

                            if (paramsSize == 0) {
                                result = gs.$invoke(methodString.get(0), new String[] {}, new Object[] {});
                            } else {
                                result = gs.$invoke(methodString.get(0),
                                        Arrays.copyOf(paramTypes.toArray(), paramTypes.size(), String[].class),
                                        params.toArray());
                            }

                            if (result == null) {
                                result = HttpStatus.OK.getReasonPhrase();
                            }

                            if (result.getClass() != String.class) {
                                response.writeWith(Mono.just(response.bufferFactory()
                                        .wrap(ByteBuffer.wrap(JSON.toJSONString(result).getBytes())))).subscribe();
                                return;
                            }

                            response.writeWith(Mono.just(
                                response.bufferFactory().wrap(ByteBuffer.wrap(((String) result).getBytes())))).subscribe();

                            return;
                        }
                    }
                } 

                result = statusCode.getReasonPhrase();
                response.writeWith(Mono.just(
                    response.bufferFactory().wrap(ByteBuffer.wrap(((String) result).getBytes())))).subscribe();
                return;
            }));
        }

    }

}